--	Controlador VGA (Adaptado do site: http://fpgaparatodos.com.br)
--	Este componente faz o controle dos sinais necessários para a operaçao
-- da interface VGA.
--
-- PS: As unidades de tempo utilizadas no projeto para fins de calculo estão todas na escala de ns

library IEEE;
use  IEEE.STD_LOGIC_1164.all;
use  IEEE.NUMERIC_STD.all;
use  IEEE.STD_LOGIC_UNSIGNED.all;


entity VGA_controlador is

	Generic(
		CLOCK_PERIOD : integer := 20  --ns, periodo do CLOCK_50MHz_i em ns (50MHz)
	);

   port( signal CLOCK_50MHz_i, reset_n_i : in std_logic;
			signal video_on_o : out std_logic;
			signal pixels_o,Linha_o : out std_logic_vector(10 downto 0);
			signal VGA_HS_o,VGA_VS_o : out std_logic);
       
end VGA_controlador;

architecture comportamento of VGA_controlador is
	
	constant H_max : std_logic_vector(10 Downto 0) := std_logic_vector(to_unsigned((31770/CLOCK_PERIOD),11)); -- dividir pelo periodo é o mesmo que multiplicar pela frequencia. 
	--valor máximo da variável cont_x, valor encontrado a partir da analise dos tempos do sincronismo horizontal
	-- 31,77us periodo para varrer uma Linha_o
	constant V_max : std_logic_vector(10 Downto 0) := std_logic_vector(to_unsigned((16770000/31770),11));   	-- vertical só incrementado no fim de uma Linha_o 
	--valor maximo da variável cont_y, valor encontrado a partir da analise dos tempos do sincronismo vertical
	-- 16,77ms periodo para varrer a tela 
	constant COLUMNS : std_logic_vector(10 Downto 0) := std_logic_vector(to_unsigned(640,11));
	constant LINES : std_logic_vector(10 Downto 0) := std_logic_vector(to_unsigned(480,11));
	
	signal cont_x,cont_y: std_logic_vector(10 Downto 0);
	signal RESET : std_logic;

	signal video_on_H, video_on_V: std_logic;

begin           

	RESET <= NOT(reset_n_i);
	video_on_o <= video_on_H and video_on_V;

	--Generate Horizontal and Vertical Timing Signals for Video Signal
	VIDEO_DISPLAY: Process (CLOCK_50MHz_i, reset)
		Begin
		if RESET = '1' then
			cont_x <= std_logic_vector(to_unsigned(0,11));
			cont_y <= std_logic_vector(to_unsigned(0,11));
			Video_on_H <= '0';
			Video_on_V <= '0';
			VGA_HS_o <= '0';
			VGA_VS_o <= '0';
		elsif rising_edge (CLOCK_50MHz_i) then
				pixels_o <= "0" & cont_x(10 downto 1); 
				-- cont_x é utilizado para descartar o último bit para dividir por 2 a frequencia
            -- De forma com que o Clock seja semelhante ao do monitor.
				Linha_o 	<= cont_y;
				
				-- cont_x conta os pixels_o (espaço utilizado+espaço nao utilizado+tempo extra para o sinal de sincronismo)
				--
				--  Contagem de pixels_o:
				--   												  <-H Sync->
				--   ------------------------------------__________
				--   0        									  1400     


				 If (cont_x >= H_max) then
					cont_x <= "00000000000";
				 Else
					cont_x <= cont_x + "00000000001";
				 End if;

				-- O VGA_HS_o deve permanecer em nível lógico alto por 26,11 us
				-- então em baixo por 3,77 us
				
				
				 If  (cont_x >= (26110/CLOCK_PERIOD)) and (cont_x <= (27060 + 3770)/CLOCK_PERIOD) Then
						VGA_HS_o <= '0';
				 ELSE
						VGA_HS_o <= '1';
				 End if;
				 
				-- Ajusta o tempo do Video_on_H
				-- video_on_H é 0 entre 25,17us até 31,77us (Periodo)
				
				 If (cont_x <= (25170/CLOCK_PERIOD)) Then  
					video_on_H <= '1';
				 ELSE
					video_on_H <= '0';
				 End if;

				--  Contagem de linhas...
				--	 Linha_o conta as linhas de pixels_o (127 + tempo extra para sinais de sincronismo)
				--
				--  <-- 				linhas utilizadas				-->  ->V Sync<-
				--  -----------------------------------------------_______------------
				--  0                          			            495-494         528

				 If (cont_y >= V_max) and (cont_x >= 736) then
						cont_y <= "00000000000";
				 Elsif (cont_x = H_Max) Then
						  cont_y <= cont_y + "00000000001";
				 End if;

				-- Generate Vertical Sync Signal (15,700ms até 15,764ms VGA_VS_o= '0')
				 If (cont_y >= (15700000/31770)) and (cont_y <= (15764000/31770)) Then  
					VGA_VS_o <= '0';
				 ELSE
					VGA_VS_o <= '1';
				 End if;
				 
				-- Ajusta o tempo do Video_on_V
				If (cont_y <= (15250000/31770)) Then
					video_on_V <= '1';
				ELSE
					video_on_V <= '0';
				End if;
				
			End if;
			
		end process VIDEO_DISPLAY;

end comportamento;

